﻿//--------------------------------------------------------------------------
// Project    : C# Project MBWordPressCopyAndPasteTool
// Environment: Visual Studio 2008, .NET Framework 3.5, Windows Application
// Date       : 2017-11-24
// Version    : 0.2
// Autor      : Manfred Becker
// Web        : http://mikrocontroller.bplaced.net/wordpress/
// Function   : Copies WordPress contens from web.archive.org URLs
//--------------------------------------------------------------------------
// 2017-11-19 : Project started
// 2017-11-24 : CopyToClipboard function added,
//              Convert XML Notofications function added
// 2017-11-25 : ContentRemove function added
// 2017-11-27 : ContentRemoveHref function added
//--------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MBWordPressCopyAndPasteTool
{
    public partial class Form1 : Form
    {
        // Please note: optional parameters in C# functions are not available in VS 2008
        // The feature was first added in .NET 4.0
        // You can work around this by using overloaded methods.

        private string GetContent(string StartTag, string EndTag)
        {
            // searching for content between StartTag and EndTag...
            string DocumentText = this.webBrowser1.DocumentText;
            string Content = "";

            int indexToStartTag = DocumentText.IndexOf(StartTag, 0);
            if (indexToStartTag >= 0)
            {
                indexToStartTag += StartTag.Length;
                int indexToEndTag = DocumentText.IndexOf(EndTag, indexToStartTag);
                int length = indexToEndTag - indexToStartTag;
                if (length > 0)
                {
                    // get content
                    Content = this.webBrowser1.DocumentText.Substring(indexToStartTag, length);
                    while (Content.StartsWith("\n") || Content.StartsWith("\t"))
                    {
                        // deleting leading CR\LF
                        Content = Content.TrimStart('\n');
                        // deleting leading tabs
                        Content = Content.TrimStart('\t');
                    }

                    while (Content.EndsWith("\n") || Content.EndsWith("\t"))
                    {
                        // deleting final tabs
                        Content = Content.TrimEnd('\t');
                        // deleting final CR\LF
                        Content = Content.TrimEnd('\n');
                    }
                }
            }

            // search and replace <pre> content </pre>
            Content = Content.Replace("<pre><code>", "<pre lang=\"c\" line=\"1\">");
            Content = Content.Replace("</code></pre>", "</pre>");
            Content = Content.Replace("<pre class=\"brush: cpp; title: ; notranslate\" title=\"\">", "<pre lang=\"c\" line=\"1\">");

            // search and replace hyperlinks
            Content = Content.Replace("../../wordpress/", "http://mikrocontroller.bplaced.net/wordpress/");

            int indexToSearchText;
            int indexToEndOfSearchText;
            string SearchText = "http://web.archive.org/web/";
            string ReplaceText = "";
            while ((indexToSearchText = Content.IndexOf(SearchText, 0)) >= 0)
            {
                // Replace("http://web.archive.org/web/20160713155030/", "");
                // Replace("http://web.archive.org/web/20160713155030im_/", "");

                indexToEndOfSearchText = Content.IndexOf("/http", indexToSearchText + SearchText.Length);

                if (indexToEndOfSearchText > 0)
                {
                    SearchText = Content.Substring(indexToSearchText, indexToEndOfSearchText - indexToSearchText + 1);
                    Content = Content.Replace(SearchText, ReplaceText);
                }
                SearchText = "http://web.archive.org/web/";
            }

            // <a class="comment-reply-link" href="/web/20170818123252/http://mikrocontroller.bplaced.net/wordpress/?page_id=46&amp;replytocom=506#respond" onclick="return addComment.moveForm(&quot;comment-506&quot;, &quot;506&quot;, &quot;respond&quot;, &quot;46&quot;)">Antworten</a>			</div><!-- .reply -->
            SearchText = "href=\"/web/";
            ReplaceText = "href=\"";
            while ((indexToSearchText = Content.IndexOf(SearchText, 0)) >= 0)
            {
                // Replace("href="/web/20170818123252/", "href="");

                indexToEndOfSearchText = Content.IndexOf("/http", indexToSearchText + SearchText.Length);

                if (indexToEndOfSearchText > 0)
                {
                    SearchText = Content.Substring(indexToSearchText, indexToEndOfSearchText - indexToSearchText + 1);
                    Content = Content.Replace(SearchText, ReplaceText);
                }
                SearchText = "href=\"/web/";
            }


            // search and replace XML notificatins
            if (checkBoxConvertXML.Checked == true)
            {
                Content = Content.Replace("&lt;", "<");
                Content = Content.Replace("&gt;", ">");
                Content = Content.Replace("&amp;", "&");
                Content = Content.Replace("&quot;", "\"");
                Content = Content.Replace("&apos;", "'");
            }

            return Content;
        }

        private string ContentRemoveHref(string Content, string RemoveStartTag, string RemoveEndTag)
        {
            //Input: <div class="comment-meta commentmetadata"><a href="http://mikrocontroller.bplaced.net/wordpress/?page_id=46#comment-506">20. Juni 2013 um 23:40</a></div><!-- .comment-meta .commentmetadata -->
            //Output:<div class="comment-meta commentmetadata">20. Juni 2013 um 23:40</div><!-- .comment-meta .commentmetadata -->
            if (Content.Length > 0 && RemoveStartTag.Length > 0 && RemoveEndTag.Length > 0)
            {
                // check if RemoveStartTag and RemoveEndTag is available then go on
                int indexToStartTag;
                int startIndex = 0;
                while ((indexToStartTag = Content.IndexOf(RemoveStartTag, startIndex)) >= 0)
                {
                    int indexToEndTag = Content.IndexOf(RemoveEndTag, indexToStartTag);
                    if (indexToEndTag >= 0)
                    {
                        int length = indexToEndTag - indexToStartTag - RemoveStartTag.Length;
                        if (length > 15) // length of <a href="">i</a>
                        {
                            int x = Content.IndexOf("<a href=", indexToStartTag + RemoveStartTag.Length);
                            int y = Content.IndexOf(">", indexToStartTag + RemoveStartTag.Length);
                            int l = y - x + 1;
                            if (l >= 11)
                            {
                                Content = Content.Remove(x, l);
                                x = Content.IndexOf("</a>", indexToStartTag + RemoveStartTag.Length);
                                if (x >= 0 && x < indexToEndTag - l)
                                {
                                    Content = Content.Remove(x, 4);
                                }
                            }
                        }
                        startIndex = Content.IndexOf(RemoveEndTag, indexToStartTag) + RemoveEndTag.Length;
                    }
                }
            }

            return Content;
        }


        private string ContentRemove(string Content, string RemoveStartTag, string RemoveEndTag)
        {
            if (Content.Length > 0 && RemoveStartTag.Length > 0 && RemoveEndTag.Length > 0)
            {
                // check if RemoveStartTag and RemoveEndTag is available then remove it
                int indexToStartTag;
                while ((indexToStartTag = Content.IndexOf(RemoveStartTag, 0)) >= 0)
                {
                    int indexToEndTag = Content.IndexOf(RemoveEndTag, indexToStartTag);
                    int length = indexToEndTag - indexToStartTag + RemoveEndTag.Length;
                    if (length > RemoveEndTag.Length)
                    {
                        Content = Content.Remove(indexToStartTag, length);
                        while (Content.StartsWith("\n") || Content.StartsWith("\t"))
                        {
                            // deleting leading CR\LF
                            Content = Content.TrimStart('\n');
                            // deleting leading tabs
                            Content = Content.TrimStart('\t');
                        }

                        while (Content.EndsWith("\n") || Content.EndsWith("\t"))
                        {
                            // deleting final tabs
                            Content = Content.TrimEnd('\t');
                            // deleting final CR\LF
                            Content = Content.TrimEnd('\n');
                        }
                    }
                }
            }

            return Content;
        }



        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Text = "MBWordPressCopyAndPasteTool Version 2017-11-27";

            //Old sources of mikrocontroller.bplaced.net
            this.textBoxURL.Text = "http://web.archive.org/web/20170714171504/http://mikrocontroller.bplaced.net/wordpress/";
            //this.textBoxURL.Text = "http://web.archive.org/web/20160701162808/http://mikrocontroller.bplaced.net/wordpress/";
            //this.textBoxURL.Text = "http://www.mikrocontroller.j--k.de/";
            //this.textBoxURL.Text = "http://www.mikrocontroller.j--k.de/alternative/wordpress/";
            this.webBrowser1.Navigate(textBoxURL.Text);
        }

        private void textBoxURL_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                this.webBrowser1.Navigate(textBoxURL.Text);
            }
        }

        private void buttonHtmlContent_Click(object sender, EventArgs e)
        {
            this.richTextBox1.Text = this.webBrowser1.DocumentText;
        }
        private void buttonEntryContent_Click(object sender, EventArgs e)
        {
            // searching for "entry-content"
            string StartTag = "<div class=\"entry-content\">";
            string EndTag = "</div><!-- .entry-content -->";

            string TextContent = GetContent(StartTag, EndTag);

            if (TextContent.Length > 0)
            {
                this.richTextBox1.Text = TextContent;
                if (checkBoxCopyToClipboard.Checked == true)
                {
                    Clipboard.SetText(TextContent);
                }
            }
        }

        private void buttonComments_Click(object sender, EventArgs e)
        {
            // searching for "comments">
            string StartTag = "<div id=\"comments\">";
            string EndTag = "</div><!-- #comments -->";
            string RemoveStartTag1 = "<div id=\"respond\"";
            string RemoveEndTag1 = "</div><!-- #respond -->";
            string RemoveStartTag2 = "<div class=\"reply\">";
            string RemoveEndTag2 = "</div><!-- .reply -->";
            string RemoveStartTag3 = "<div class=\"comment-meta commentmetadata\">";
            string RemoveEndTag3 = "<!-- .comment-meta .commentmetadata -->";
            string TextContent;

            //string TextContent = GetContent(StartTag, EndTag, RemoveStartTag1, RemoveEndTag1, RemoveStartTag2, RemoveEndTag2);
            TextContent = GetContent(StartTag, EndTag);
            TextContent = ContentRemove(TextContent, RemoveStartTag1, RemoveEndTag1);
            TextContent = ContentRemove(TextContent, RemoveStartTag2, RemoveEndTag2);
            TextContent = ContentRemoveHref(TextContent, RemoveStartTag3, RemoveEndTag3);
            TextContent = TextContent.Replace("\t", "");

            if (TextContent.Length > 0)
            {
                this.richTextBox1.Text = TextContent;
                if (checkBoxCopyToClipboard.Checked == true)
                {
                    Clipboard.SetText(TextContent);
                }
            }
        }

        private void buttonURL1_Click(object sender, EventArgs e)
        {
            //Old sources of mikrocontroller.bplaced.net
            this.textBoxURL.Text = "http://web.archive.org/web/20170714171504/http://mikrocontroller.bplaced.net/wordpress/";
            this.webBrowser1.Navigate(textBoxURL.Text);
        }

        private void buttonURL2_Click(object sender, EventArgs e)
        {
            //Old sources of mikrocontroller.bplaced.net
            this.textBoxURL.Text = "http://www.mikrocontroller.j--k.de/";
            this.webBrowser1.Navigate(textBoxURL.Text);
        }

        private void buttonURL3_Click(object sender, EventArgs e)
        {
            //Old sources of mikrocontroller.bplaced.net
            this.textBoxURL.Text = "http://www.mikrocontroller.j--k.de/alternative/wordpress/";
            this.webBrowser1.Navigate(textBoxURL.Text);
        }

        private void buttonURL4_Click(object sender, EventArgs e)
        {
            this.textBoxURL.Text = "http://mikrocontroller.bplaced.net/wordpress/";
            this.webBrowser1.Navigate(textBoxURL.Text);
        }

        private void webBrowser1_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            // read URL
            string Url = webBrowser1.Url.AbsoluteUri;
            this.textBoxURL.Text = Url;

            // get DocumentText (F12)
            this.richTextBox1.Text = this.webBrowser1.DocumentText;

            // searching for "entry-content" ...
            string StartTag = "<div class=\"entry-content\">";
            string EndTag = "</div><!-- .entry-content -->";
            string TextContent = GetContent(StartTag, EndTag);

            // ... if found, enable Content Button
            if (TextContent.Length > 0)
                this.buttonEntryContent.Enabled = true;
            else
                this.buttonEntryContent.Enabled = false;


            // searching for "comments">
            StartTag = "<div id=\"comments\">";
            EndTag = "</div><!-- #comments -->";
            TextContent = GetContent(StartTag, EndTag);

            // ... if found, enable Comments Button
            if (TextContent.Length > 0)
                this.buttonComments.Enabled = true;
            else
                this.buttonComments.Enabled = false;

        }

        private void webBrowser1_Navigating(object sender, WebBrowserNavigatingEventArgs e)
        {
            this.richTextBox1.Text = "navigating, please wait...";
            this.buttonEntryContent.Enabled = false;
            this.buttonComments.Enabled = false;
        }

    }
}
